package triggerpointui.views;


import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.jdt.internal.ui.callhierarchy.TreeRoot;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.*;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.SWT;

import contextualbreakpoints.breakpoints.JavaContextualLineBreakpoint;




/**
 * This sample class demonstrates how to plug-in a new
 * workbench view. The view shows data obtained from the
 * model. The sample creates a dummy model on the fly,
 * but a real implementation would connect to the model
 * available either in this or another plug-in (e.g. the workspace).
 * The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model
 * objects should be presented in the view. Each
 * view can present the same model objects using
 * different labels and icons, if needed. Alternatively,
 * a single label provider can be shared between views
 * in order to ensure that objects of the same type are
 * presented in the same way everywhere.
 * <p>
 */

public class TriggerPointHierarchyView extends ViewPart {
	private TreeViewer viewer;
	private Action refreshViewAction;
	private Action deleteAction;
	private Action doubleClickAction;
	private Action toggleTriggerAction;
	

	class NameSorter extends ViewerSorter {
	}

	/**
	 * The constructor.
	 */
	public TriggerPointHierarchyView() {
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.setContentProvider(new TriggerPointHierarchyViewContentProvider());
		viewer.setLabelProvider(new TriggerPointHierarchyViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(getViewSite());
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
	}


	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				TriggerPointHierarchyView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(refreshViewAction);
		manager.add(new Separator());
		manager.add(deleteAction);
	}

	private void fillContextMenu(IMenuManager manager) {
		//manager.add(refreshViewAction);
		manager.add(deleteAction);
		manager.add(toggleTriggerAction);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(refreshViewAction);
//		manager.add(action2);
	}

	private void makeActions() {
		// refresh the trigger hierarchy tree
		refreshViewAction = new Action() {
			public void run() {
				viewer.refresh();
				//showMessage("Trigger Hierarchy Refreshed");
			}
		};
		refreshViewAction.setText("Refresh");
		refreshViewAction.setToolTipText("Refresh the Trigger hierarchy view");
		refreshViewAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_TOOL_REDO));
		
		// delete the selected breakpoint
		deleteAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				try {
					DebugPlugin.getDefault().getBreakpointManager().removeBreakpoint((JavaContextualLineBreakpoint)obj, true);
				} catch (CoreException e) {
					showMessage("Problem deleting breakpoint");
				}
				viewer.refresh();
				showMessage("Breakpoint deleted");
			}
		};
		
		deleteAction.setText("Delete Breakpoint");
		deleteAction.setToolTipText("Delete the selected breakpoint");
		deleteAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
				getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		
		// toggle trigger only of the selected breakpoint
		toggleTriggerAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				JavaContextualLineBreakpoint bp = (JavaContextualLineBreakpoint)obj;
				if (bp != null) {
					if (bp.isShadowOnly()) {
						bp.setShadowOnly(false);
					} else {
						bp.setShadowOnly(true);
					}
				}
				viewer.refresh();
				//showMessage("Breakpoint deleted");
			}
		};
		toggleTriggerAction.setText("Toggle trigger only");
		toggleTriggerAction.setToolTipText("Toggle whether breakpoint is trigger only");
		toggleTriggerAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
				getImageDescriptor(ISharedImages.IMG_OBJS_BKMRK_TSK));
		
		
		
		// doubleclick on a breakpoint - show properties
		doubleClickAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				//showMessage("Double-click detected on "+obj.toString());
			}
		};
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}
	
	private void showMessage(String message) {
		MessageDialog.openInformation(
			viewer.getControl().getShell(),
			"Triggerpoint Hierarchy View",
			message);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
}