package contextualbreakpoints.breakpoints;


import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaBreakpoint;
import org.eclipse.jdt.internal.debug.core.breakpoints.JavaLineBreakpoint;
import org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget;
import org.eclipse.jdt.internal.debug.core.model.JDIThread;

import com.sun.jdi.event.Event;

import contextualbreakpoints.contextlogic.DebuggerContext;
import contextualbreakpoints.contextlogic.DefaultsLogic;
import contextualbreakpoints.pointcuts.Pointcut;
/**
 * @author Rick 
 * 
 * Like a JavaLineBreakpoint except it is contextual and also serves as context
 *
 */
public class JavaContextualLineBreakpoint extends JavaLineBreakpoint implements IJavaContextualLineBreakpoint {
	
	private boolean isTriggerOnly; 
	
	// FIXME: change the marker
	private static final String JAVA_LINE_BREAKPOINT = "org.eclipse.jdt.debug.javaLineBreakpointMarker"; //$NON-NLS-1$

	// true if this breakpoint has been reached
	private boolean hasBeenReached;
	
	private boolean hasSuspended;
	
	// true if this breakpoint has ever suspended *since creation*
	private boolean hasEverSuspended;
	
	// the context condition 
	private Pointcut contextCondition; 
	
	private int savedLineNumber; // line number before delete
	private String savedTypeName; // typename before delete
	private int savedCharStart; // char start before delete
	private int savedCharEnd; // char end before delete
	
	// saved contexts
	private List savedContexts; 
	private List savedCallStack;
	private List savedBeforeBps;
	private List savedAfterBps;
	
	/**
	 * Constructor - create a new triggered breakpoint
	 */
	public JavaContextualLineBreakpoint(IResource resource, String typeName, int lineNumber, int charStart, int charEnd, int hitCount, boolean add, Map attributes) throws DebugException {
		super(resource, typeName, lineNumber, charStart, charEnd, hitCount, add, attributes, JAVA_LINE_BREAKPOINT);
		
		// save some info for later use when breakpoint is deactivated
		savedLineNumber = lineNumber;
		savedTypeName = typeName;
		savedCharStart = charStart;
		savedCharEnd = charEnd;
		
		contextCondition = DefaultsLogic.getDefaultResidual();
		setShadowOnly(false);
		initializeContext();
		
		savedContexts = new ArrayList();
	}
	
	
	/**
	 * There are some special cases during breakpoint creation
	 */
	private void initializeContext() {
		resetContext();
		setHasEverSuspended(false);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#handleBreakpointEvent(com.sun.jdi.event.Event, org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget, org.eclipse.jdt.internal.debug.core.model.JDIThread)
	 */
	public boolean handleBreakpointEvent(Event event, JDIDebugTarget target, JDIThread thread) {
		// if it's only a shadow, don't do anything
		if (isShadowOnly()) {
			return true;
		}
		
		// used for before and after conditions
		hasBeenReached = true;
		 
		if ((contextCondition.isMatch(thread) && !isTriggerOnly)) {
			if (super.handleBreakpointEvent(event, target, thread)) {
				return true;
			}
			hasSuspended = true;
			return false;
		}

		// otherwise resume
		return true;
	}
	
	/**
	 * @param event
	 * @param target
	 * @param thread
	 * @return true iff contextCondition doesn't match
	 */
	public boolean fakeHandleBreakpointEvent(Event event, JDIDebugTarget target, JDIThread thread) {
		if (isShadowOnly()) {
			return true;
		}
		if ((contextCondition.isMatch(thread) && !isTriggerOnly)) {
			return false;
		}
		return true;
	}
	
	/* (non-Javadoc)
	 * @see triggerpointmanagement.IJavaTriggeredBreakpoint#reset()
	 */
	public void resetContext() {
		hasBeenReached = false;
		hasSuspended = false;
	}

	public void setContextCondition(Pointcut contextCondition) {
		this.contextCondition = contextCondition;
	}
	
	public void setShadowOnly(boolean yes) {
		isTriggerOnly = yes;
	}

	public boolean isShadowOnly() {
		return isTriggerOnly;
	}

	public Pointcut getContextCondition() {
		return contextCondition;
	}

	public boolean hasBeenReached() {
		return hasBeenReached;
	}

	public boolean hasSuspended() {
		return hasSuspended;
	}
	
	/**
	 * PRE: Marker must not be deleted yet
	 */
	private void saveInfo() {
		try {
			savedLineNumber = getLineNumber();
			savedCharEnd = getCharEnd();
			savedCharStart = getCharStart();
			savedTypeName = getTypeName();
		} catch (CoreException e) {
			System.err.println("something wrong with saving info...");
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.model.IBreakpoint#delete()
	 */
	public void delete() throws CoreException {
		
		// NOTE: must call saveInfo() BEFORE super.delete()
		saveInfo();
		
		super.delete();
		
		// and now, if it's not registered, make it a shadow
		setShadowOnly(true);
		
		// and re-register with the VM if debugging session is active
		if (DebuggerContext.getInstance().getDebuggerActive()) {
			JDIDebugTarget targ = DebuggerContext.getInstance().getMostRecentTarget();
			if (targ != null) {
				addToTarget(targ);
			}
		}
		

		// now for our own stuff - if WE get deregistered, then make sure any
		// LineExecution pcs in our residual expression get deregistered from the 
		// pointcut manager
		deregisterPointcuts();
	}
	
	
	private void deregisterPointcuts() {
		// walk through residual expression ...... for now we can ignore doing this
		// but we should do this later to avoid memory leaks
	}
	
	public boolean hasEverSuspended() {
		return hasEverSuspended;
	}
	
	public void setHasEverSuspended(boolean b) {
		hasEverSuspended = b;
	}

	public void setHasBeenReached(boolean reached) {
		hasBeenReached = reached;
		
	}

	/*
	 * **** The following methods override those in superclasses to deal with situations
	 *      when the marker no longer exists (e.g. after breakpoint has been deactivated)
	 *      
	 */

	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.model.ILineBreakpoint#getLineNumber()
	 * Overridees to deal with non-existent marker
	 */
	public int getLineNumber() throws CoreException {
		try {
			return super.getLineNumber();
		} catch (DebugException e) {
			return savedLineNumber;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.model.ILineBreakpoint#getCharStart()
	 */
	public int getCharStart() throws CoreException {
		try {
			return super.getCharStart();
		} catch (DebugException e) {
			return savedCharStart;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.model.ILineBreakpoint#getCharEnd()
	 */
	public int getCharEnd() throws CoreException {
		try {
			return super.getCharEnd();
		} catch (DebugException e) {
			return savedCharEnd;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getTypeName()
	 * Overrides to deal with non-existent marker
	 */
	public String getTypeName() throws CoreException {
		try {
			return super.getTypeName();
		} catch (DebugException e) {
			return savedTypeName;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getSuspendPolicy()
	 * Overrides to deal with non-existent marker
	 */
	public int getSuspendPolicy() throws CoreException {
		try {
			return super.getSuspendPolicy();
		} catch (DebugException e) {
			// default; we're not going to suspend anyways
			return IJavaBreakpoint.SUSPEND_THREAD;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getHitCount()
	 * Overrides to deal with non-existent marker
	 */
	public int getHitCount() throws CoreException {
		try {
			return super.getHitCount();
		} catch (DebugException e) {
			// default
			return -1;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#getInstallCount()
	 */
	public int getInstallCount() throws CoreException {
		try {
			return super.getInstallCount();
		} catch (DebugException e) {
			// default + 1
			return 1;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#incrementInstallCount()
	 */
	protected void incrementInstallCount() throws CoreException {
		try {
			super.incrementInstallCount();
			return;
		} catch (DebugException e) {
			// just don't do anything - getInstallCount returns 1 anyways
			return;
		}
	}
	
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.debug.core.model.IBreakpoint#isEnabled()
	 */
	public boolean isEnabled() {
		return true; // contextuallinebreakpoints always have shadows so we always
		             // need to get events
		             // we deal with enable state in handleBreakpointEvent
	}


	public void setSavedContext(List pc) {
		savedContexts = pc;
	}


	public List getSavedContext() {
		return savedContexts;
	}
	
	
	
	public String toString() {
		try {
			return "JavaContextualLineBreakpoint: " + getTypeName() + " - " + getLineNumber(); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (CoreException e) {
			return super.toString();
		}
	}


	/**
	 * @return Returns the savedCallStack.
	 */
	public List getSavedCallStack() {
		return savedCallStack;
	}


	/**
	 * @param savedCallStack The savedCallStack to set.
	 */
	public void setSavedCallStack(List savedCallStack) {
		this.savedCallStack = savedCallStack;
	}


	public void setSavedBeforeBreakpoints(List bps) {
		savedBeforeBps = bps;
	}


	public List getSavedBeforeBreakpoints() {
		return savedBeforeBps;
	}


	public void setSavedAfterBreakpoints(List bps) {
		savedAfterBps = bps;
	}


	public List getSavedAfterBreakpoints() {
		return savedAfterBps;
	}
	
	
	

}
